@@ -224,6 +224,32 @@ end
         end
     end
 end
+@testitem "detect duplicate LineNumberNode in @stable expansion" begin
+    using DispatchDoctor
+
+    # Basically we want to verify that the simulator function
+    # doesn't duplicate any LineNumberNodes, which can
+    # mess with stacktraces and coverage.
+    ex = @macroexpand(@stable(function f(x)
+        return x
+    end))
+
+    function collect_line_nodes!(line_nodes, expr)
+        if expr isa LineNumberNode
+            push!(line_nodes, expr)
+        elseif expr isa Expr
+            for arg in expr.args
+                collect_line_nodes!(line_nodes, arg)
+            end
+        end
+        return line_nodes
+    end
+
+    line_nodes = collect_line_nodes!([], ex)
+
+    @test length(line_nodes) > 1
+    @test length(unique(line_nodes)) == length(line_nodes)  # No dupes!
+end
 @testitem "Type specialization" begin
     using DispatchDoctor
     for codegen_level in ("debug", "min")
@@ -1325,5 +1351,39 @@ end
     DispatchDoctor.JULIA_OK &&
         @test_throws TypeInstabilityError tuple_from_vector2([1, 2, 3])
 end
+@testitem "issue with specializing_typeof on unbound type parameters" begin
+    using DispatchDoctor
+
+    @stable function test_function(arg::Type{<:Type})
+        return nothing
+    end
+
+    # This should trigger the error
+    P = (Type{T} where {T}).body
+    @test test_function(P) === nothing
+end
+@testitem "issue with closures causing instability" begin
+    using DispatchDoctor
+
+    @stable function f(x)
+        @stable g() = x
+        return g
+    end
+
+    @test_nowarn f(1.0)()
+    @test f(1.0)() == 1.0
+end
+@testitem "issue with nested allow_unstable" begin
+    using DispatchDoctor
+
+    @stable f() = Val(rand())
+
+    # Issue was that the `allow_unstable` would set the
+    # `INSTABILITY_CHECK_ENABLED` to `false` and then the
+    # `f` would throw an error, _even though_ we are still
+    # within another `allow_unstable` block.
+    @test_nowarn allow_unstable(() -> (allow_unstable(f); f()))
+    @test_throws TypeInstabilityError f()
+end
 
 @run_package_tests
